import 'package:flutter/material.dart';
import 'package:photo/src/entity/options.dart';
import 'package:photo/src/provider/i18n_provider.dart';
import 'package:photo_manager/photo_manager.dart';
import 'package:photo/src/provider/gallery_list_provider.dart';
import 'package:photo/src/provider/selected_provider.dart';
import 'package:photo/src/provider/asset_provider.dart';
import 'package:photo/src/provider/config_provider.dart';

import 'photo_main_page.dart';
import 'photo_preview_page.dart';


class PhotoListPage extends StatefulWidget {
  final ValueChanged<List<AssetEntity>> onSelected;
  final ListOptions options;
  final List<AssetPathEntity> photoList;

  const PhotoListPage({
    Key key,
    this.onSelected,
    this.options,
    this.photoList,
  }) : super(key: key);

  @override
  _PhotoListPageState createState() => _PhotoListPageState();
}

class _PhotoListPageState extends State<PhotoListPage>
    with SelectedProvider, GalleryListProvider, WidgetsBindingObserver {
  ListOptions get options => widget.options;
  I18nProvider get i18nProvider => PhotoPickerProvider.of(context).provider;
  AssetProvider get assetProvider => PhotoPickerProvider.of(context).assetProvider;
  List<AssetEntity> get list => assetProvider.data;

  bool _isInit = false;

  GlobalKey scaffoldKey;
  ScrollController scrollController;

  bool isPushed = false;

  // Throttle _changeThrottle;
  
  @override
  void initState() {
    super.initState();
    scaffoldKey = GlobalKey();
    scrollController = ScrollController();
    // WidgetsBinding.instance.addPostFrameCallback((timeStamp) async {
    //   var requestPermission = await PhotoManager.requestPermission();
    //   if (requestPermission != true) {
    //     var result = await showDialog(
    //       context: context,
    //       builder: (ctx) => NotPermissionDialog(
    //         provider.getNotPermissionText(),
    //       ),
    //     );
    //     if (result == true) {
    //       PhotoManager.openSetting();
    //     }
    //     return null;
    //   }
    // });
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    if (!_isInit) {
      final pickedList = PhotoPickerProvider.of(context).pickedAssetList ?? [];
      addPickedAsset(pickedList.toList());
      _refreshList();
    }
  }

  @override
  void dispose() {
    scaffoldKey = null;
    super.dispose();
  }

  void _refreshList() async {
    await Future.delayed(Duration.zero);

    _refreshListFromGallery();
  }

  Future<void> _refreshListFromGallery() async {
    List<AssetPathEntity> pathList;
    switch (options.pickType) {
      case PickType.onlyImage:
        pathList = await PhotoManager.getAssetPathList(type: RequestType.image, onlyAll: true);
        break;
      case PickType.onlyVideo:
        pathList = await PhotoManager.getAssetPathList(type: RequestType.video, onlyAll: true);
        break;
      default:
        pathList = await PhotoManager.getAssetPathList(type: RequestType.image | RequestType.video, onlyAll: true);
    }

    return _onRefreshAssetPathList(pathList);
  }

  Future<void> _onRefreshAssetPathList(List<AssetPathEntity> pathList) async {
    if (pathList == null) {
      return;
    }

    options.sortDelegate.sort(pathList);

    galleryPathList.clear();
    galleryPathList.addAll(pathList);

    if (pathList.isNotEmpty) {
      assetProvider.current = pathList[0];
      await assetProvider.loadMore();
    }

    for (var path in pathList) {
      if (path.isAll) {
        path.name = i18nProvider.getAllGalleryText(options);
      }
    }

    setState(() {
      _isInit = true;
    });
  }

  @override
  bool isUpperLimit() {
    var result = selectedCount == options.maxSelected;
    if (result) _showTip(i18nProvider.getMaxTipText(options));
    return result;
  }

  @override
  void sure() {
    widget.onSelected?.call(selectedList);
  }

  void _showTip(String msg) {
    if (isPushed) {
      return;
    }
    Scaffold.of(scaffoldKey.currentContext).showSnackBar(
      SnackBar(
        content: Text(
          msg,
          style: options.theme.textStyle.copyWith(color: options.theme.onPrimaryColor),
        ),
        duration: Duration(milliseconds: 1500),
        backgroundColor: options.theme.primaryColor.withOpacity(0.7),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    if (!_isInit) {
      return _buildLoading();
    }
    final noMore = assetProvider.noMore;

    final count = assetProvider.count + (noMore ? 0 : 1);

    return ListView.builder(
      scrollDirection: options.scrollDirection,
      physics: ClampingScrollPhysics(),
      itemCount: count,
      itemExtent: options.itemWidth,
      itemBuilder: _buildItem,
    );
  }

  _loadMore() async {
    await assetProvider.loadMore();
    setState(() {});
  }

  Widget _buildItem(BuildContext context, int index) {
    final noMore = assetProvider.noMore;
    if (!noMore && index == assetProvider.count) {
      _loadMore();
      return _buildLoading();
    }

    var data = list[index];
    return RepaintBoundary(
      child: GestureDetector(
        onTap: () => _onItemClick(data, index),
        child: Stack(
          children: <Widget>[
            ImageItem(
              entity: data,
              themeColor: options.theme.primaryColor,
              size: options.thumbSize,
              loadingDelegate: options.loadingDelegate,
              badgeDelegate: options.badgeDelegate,
            ),
            _buildMask(containsEntity(data)),
            _buildSelected(data),
          ],
        ),
      ),
    );
  }

  _buildMask(bool showMask) {
    return IgnorePointer(
      child: AnimatedContainer(
        color: showMask ? Colors.black.withOpacity(0.5) : Colors.transparent,
        duration: Duration(milliseconds: 300),
      ),
    );
  }

  Widget _buildSelected(AssetEntity entity) {
    var currentSelected = containsEntity(entity);
    return Positioned(
      right: 0.0,
      width: 36.0,
      height: 36.0,
      child: GestureDetector(
        onTap: () {
          changeCheck(!currentSelected, entity);
        },
        behavior: HitTestBehavior.translucent,
        child: _buildText(entity),
      ),
    );
  }

  void changeCheck(bool value, AssetEntity entity) {
    if (value) {
      addSelectEntity(entity);
    } else {
      removeSelectEntity(entity);
    }
    sure();
    setState(() {});
  }

  Widget _buildText(AssetEntity entity) {
    var isSelected = containsEntity(entity);
    Widget child;
    BoxDecoration decoration = BoxDecoration(borderRadius: BorderRadius.circular(1.0),);
    if (isSelected) {
      child = Text(
        (indexOfSelected(entity) + 1).toString(),
        textAlign: TextAlign.center,
        style: options.theme.textStyle.apply(
          fontSizeDelta: -4,
          color: options.theme.onPrimaryColor,
        ),
      );
      decoration = decoration.copyWith(color: options.theme.primaryColor);
    } else {
      decoration = decoration.copyWith(border: Border.all(
        color: options.theme.onPrimaryColor.withAlpha(200),
      ),);
    }
    return Padding(
      padding: const EdgeInsets.all(8.0),
      child: AnimatedContainer(
        duration: Duration(milliseconds: 300),
        decoration: decoration,
        alignment: Alignment.center,
        child: child,
      ),
    );
  }

  void _onItemClick(AssetEntity data, int index) {
    var result = PhotoPreviewResult();
    isPushed = true;
    Navigator.of(context).push(
      MaterialPageRoute(
        builder: (ctx) {
          return PhotoPickerProvider(
            provider: PhotoPickerProvider.of(context).provider,
            options: options,
            child: PhotoPreviewPage(
              selectedProvider: this,
              list: List.of(list),
              initIndex: index,
              changeProviderOnCheckChange: true,
              result: result,
              isPreview: false,
              assetProvider: assetProvider,
            ),
          );
        },
      ),
    ).then((v) {
      if (handlePreviewResult(v)) {
        Navigator.pop(context, v);
        return;
      }
      isPushed = false;
      setState(() {});
    });
  }

  bool handlePreviewResult(List<AssetEntity> v) {
    if (v == null) {
      return false;
    }
    if (v is List<AssetEntity>) {
      return true;
    }
    return false;
  }

  Widget _buildLoading() {
    return Center(
      child: Column(
        children: <Widget>[
          Container(
            width: 40.0,
            height: 40.0,
            padding: const EdgeInsets.all(5.0),
            child: CircularProgressIndicator(
              valueColor: AlwaysStoppedAnimation(Colors.black),
            ),
          ),
          Padding(
            padding: const EdgeInsets.all(8.0),
            child: Text(
              i18nProvider.loadingText(),
              style: const TextStyle(
                fontSize: 12.0,
              ),
            ),
          ),
        ],
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.center,
      ),
    );
  }
}